/*
 * PersonPanel.java
 * 
 * Copyright (c) 2004 ICEGT, Georgia Institute of Technology, Jeremy Goecks
 */
package edu.gatech.icegt.guesswho;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * A panel that shows a person in the GuessWhoGame application. Panel shows person's
 * picture and name.
 * 
 * @author Jeremy Goecks
 * @version 0.1
 */
public class PersonPanel extends JPanel implements MouseListener {
	
	/** Width of person's image in panel should be this. */
	protected static final int SCALED_IMAGE_WIDTH = 64;
	
	/** Image file to display if person is not a match. */
	protected static String noMatchImageFile;
	
	/** Directory where person image files live. */
	protected static String personImagesDir;
	
	/**
	 * Set attribute noMatchImageFile
	 */
	public static void setNoMatchImageFile(String noMatchImageFile) {
		PersonPanel.noMatchImageFile = noMatchImageFile;
	} // end setImageFileIfNotAMatch()
	
	/**
	 * Set attribute personImagesDir
	 */
	public static void setPersonImagesDir(String personImagesDir) {
		PersonPanel.personImagesDir = personImagesDir;
	} // end setPersonImagesDir()
	
	/** PlayGameFrame that panel is associated with. */
	protected PlayGameFrame playGameFrame;
	
	/** Person associated with panel. */
	protected Person person;
	
	/** 
	 * Is person a match with the current criteria? If yes, person's image is
	 * displayed; if not, "no match" image file is displayed.
	 */
	protected boolean isMatch;
	
	/** Label that holds panel's image. */
	protected JLabel imageLabel;
	
	/** Label that show's person's name. */
	protected JLabel nameLabel;
	
	/** 
	 * Create panel. 
	 */
	public PersonPanel(PlayGameFrame playGameFrame, Person person, boolean isMatch) {
		//
		// Init.
		//
		this.playGameFrame = playGameFrame;
		this.person = person;
		this.isMatch = isMatch;
		
		//
		// Create panel contents.
		//
		
		// Use a GridBagLayout for panel.
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(gbl);
		
		// Set constraints for person's picture.
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.gridx = 0;
		gbc.gridy = 0;
		
		// Create ImageIcon, label and add label to panel.
		File imageFile = 
			new File( 
				(isMatch ? personImagesDir + File.separator + person.getPhotoFile() : 
					noMatchImageFile)
					 );
		ImageIcon imageIcon = 
			GuessWhoGame.getImageIconWithScaledImage(imageFile, SCALED_IMAGE_WIDTH, this);
		imageLabel = new JLabel(imageIcon);
		imageLabel.setSize(10, 10);
		gbl.setConstraints(imageLabel, gbc);
		add(imageLabel);
		
		// Set constraints for person's name.
		gbc.gridy = 1;
		
		// Add label to panel.
		nameLabel = new JLabel(person.getName());
		gbl.setConstraints(nameLabel, gbc);
		add(nameLabel);
		
		// Set tooltip for panel.
		setToolTipText("<html>" +
				       "gender: " + person.getGender() + "<br>" + 
					   "hair color: " + person.getHairColor() + "<br>" + 
					   "eye color: " + person.getEyeColor() + "<br>" +
					   "earrings? " + person.hasEarrings() +
						"</html>");
		
		// Add object as mouse listener for itself.
		addMouseListener(this);
	} // end PersonPanel()
	
	/** 
	 * Returns panel's person.
	 */
	public Person getPerson() {
		return person;
	} // end getPerson()
	
	/**
	 * Set panel's person.
	 * @param person a person
	 */
	public void setPerson(Person person) {
		//
		// Set attribute.
		//
		this.person = person;
		
		//
		// Update panel: image, label, tool tip.
		//
		String imageFile = 
			(isMatch ? personImagesDir + File.separator + person.getPhotoFile() : noMatchImageFile);
		ImageIcon imageIcon = 
			GuessWhoGame.getImageIconWithScaledImage(imageFile, SCALED_IMAGE_WIDTH, this);
		imageLabel.setIcon(imageIcon);
		nameLabel.setText(person.getName());
		setToolTipText("<html>" +
				       "gender: " + person.getGender() + "<br>" + 
					   "hair color: " + person.getHairColor() + "<br>" + 
					   "eye color: " + person.getEyeColor() + "<br>" +
					   "earrings? " + person.hasEarrings() +
						"</html>");
	} // end setPerson()
	
	/**
	 * Method sets attribute isMatch and, if necessary, update's panel image.
	 * @param newVal new value for isMatch  
	 */
	public void setIsMatch(boolean newVal) {
		//
		// If new value is different from the current value, update isMatch and
		// panel's picture.
		//
		if (isMatch != newVal) {
			// Set isMatch.
			isMatch = newVal;
			
			// Update panel image.
			String imageFile = 
				(isMatch ? personImagesDir + File.separator + person.getPhotoFile() : noMatchImageFile);
			ImageIcon icon = 
				GuessWhoGame.getImageIconWithScaledImage(imageFile, SCALED_IMAGE_WIDTH, this);
			imageLabel.setIcon(icon);
		} // end if
	} // end setIsMatch()
	
	/**
	 * Indicates that user wants to guess this person.
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
		// User guessed this person.
		playGameFrame.userGuessed(person);
	} // end mouseClicked()
	
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
	}
} // end class PersonPanel

